"""
  TheengsGateway - Decode things and devices and publish data to an MQTT broker

    Copyright: (c)Florian ROBERT

    This file is part of TheengsGateway.

    TheengsGateway is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TheengsGateway is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

# python 3.6
# encoding=utf8

import json
import re

from ._decoder import getProperties
from .ble_gateway import Gateway, logger

ha_dev_classes = [
    "battery",
    "carbon_monoxide",
    "carbon_dioxide",
    "humidity",
    "illuminance",
    "signal_strength",
    "temperature",
    "timestamp",
    "pressure",
    "power",
    "current",
    "energy",
    "power_factor",
    "voltage",
]

ha_dev_units = [
    "W",
    "kW",
    "V",
    "A",
    "W",
    "°C",
    "°F",
    "ms",
    "s",
    "hPa",
    "kg",
    "lb",
    "µS/cm",
    "lx",
    "%",
    "dB",
    "B",
]


class DiscoveryGateway(Gateway):
    """BLE to MQTT gateway class with Home Assistant MQTT discovery."""

    def __init__(
        self,
        broker,
        port,
        username,
        password,
        adapter,
        scanning_mode,
        discovery_topic,
        discovery_device_name,
        discovery_filter,
        hass_discovery,
    ):
        super().__init__(
            broker, port, username, password, adapter, scanning_mode
        )
        self.discovery_topic = discovery_topic
        self.discovery_device_name = discovery_device_name
        self.discovered_entities = []
        self.discovery_filter = discovery_filter
        self.hass_discovery = hass_discovery

    def publish_device_info(self, pub_device):
        """Publish sensor directly to Home Assistant via MQTT discovery."""
        pub_device_uuid = pub_device["id"].replace(":", "")
        device_data = json.dumps(pub_device)
        if (
            pub_device_uuid in self.discovered_entities
            or pub_device["model_id"] in self.discovery_filter
        ):
            logger.debug("Already discovered or filtered: %s", pub_device_uuid)
            self.publish(device_data, self.pub_topic + "/" + pub_device_uuid)
            return

        logger.info("publishing device `%s`", pub_device)
        pub_device["properties"] = json.loads(
            getProperties(pub_device["model_id"])
        )["properties"]

        hadevice = {}
        hadevice["identifiers"] = list({pub_device_uuid})
        hadevice["connections"] = [list(("mac", pub_device_uuid))]
        hadevice["manufacturer"] = pub_device["brand"]
        hadevice["model"] = pub_device["model_id"]
        if "name" in pub_device:
            hadevice["name"] = pub_device["name"]
        else:
            hadevice["name"] = pub_device["model"]
        hadevice["via_device"] = self.discovery_device_name

        discovery_topic = self.discovery_topic + "/" + pub_device_uuid
        state_topic = self.pub_topic + "/" + pub_device_uuid
        state_topic = re.sub(
            r".+?/", "+/", state_topic, len(re.findall(r"/", state_topic)) - 1
        )
        data = getProperties(pub_device["model_id"])
        data = json.loads(data)
        data = data["properties"]

        for k in data.keys():
            device = {}
            device["stat_t"] = state_topic
            if k in pub_device["properties"]:
                if pub_device["properties"][k]["name"] in ha_dev_classes:
                    device["dev_cla"] = pub_device["properties"][k]["name"]
                if pub_device["properties"][k]["unit"] in ha_dev_units:
                    device["unit_of_meas"] = pub_device["properties"][k][
                        "unit"
                    ]
            device["name"] = pub_device["model_id"] + "-" + k
            device["uniq_id"] = pub_device_uuid + "-" + k
            if self.hass_discovery == 1:
                device["val_tpl"] = "{{ value_json." + k + " | is_defined }}"
            else:
                device["val_tpl"] = "{{ value_json." + k + " }}"
            device["state_class"] = "measurement"
            config_topic = discovery_topic + "-" + k + "/config"
            device["device"] = hadevice
            self.publish(json.dumps(device), config_topic, True)

        self.discovered_entities.append(pub_device_uuid)
        self.publish(device_data, self.pub_topic + "/" + pub_device_uuid)



"""
  TheengsGateway - Decode things and devices and publish data to an MQTT broker

    Copyright: (c)Florian ROBERT

    This file is part of TheengsGateway.

    TheengsGateway is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TheengsGateway is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import argparse
import json
import os
import sys

from .ble_gateway import run

default_config = {
    "host": "",
    "port": 1883,
    "user": "",
    "pass": "",
    "ble_scan_time": 5,
    "ble_time_between_scans": 5,
    "publish_topic": "home/TheengsGateway/BTtoMQTT",
    "subscribe_topic": "home/+/BTtoMQTT/undecoded",
    "log_level": "WARNING",
    "discovery": 1,
    "hass_discovery": 1,
    "discovery_topic": "homeassistant/sensor",
    "discovery_device_name": "TheengsGateway",
    "discovery_filter": ["IBEACON", "GAEN", "MS-CDP"],
    "adapter": "",
    "scanning_mode": "active",
    "time_sync": [],
    "time_format": 0,
}

conf_path = os.path.expanduser("~") + "/theengsgw.conf"

parser = argparse.ArgumentParser()
parser.add_argument(
    "-H", "--host", dest="host", type=str, help="MQTT host address"
)
parser.add_argument(
    "-P", "--port", dest="port", type=int, help="MQTT host port"
)
parser.add_argument(
    "-u", "--user", dest="user", type=str, help="MQTT username"
)
parser.add_argument("-p", "--pass", dest="pwd", type=str, help="MQTT password")
parser.add_argument(
    "-pt", "--pub_topic", dest="pub_topic", type=str, help="MQTT publish topic"
)
parser.add_argument(
    "-st",
    "--sub_topic",
    dest="sub_topic",
    type=str,
    help="MQTT subscribe topic",
)
parser.add_argument(
    "-pa",
    "--publish_all",
    dest="publish_all",
    type=int,
    help="Enable(1) or disable(0) publishing of all beacons",
)
parser.add_argument(
    "-sd",
    "--scan_duration",
    dest="scan_dur",
    type=int,
    help="BLE scan duration (seconds)",
)
parser.add_argument(
    "-tb",
    "--time_between",
    dest="time_between",
    type=int,
    help="Seconds to wait between scans",
)
parser.add_argument(
    "-ll",
    "--log_level",
    dest="log_level",
    type=str,
    help="TheengsGateway log level",
    choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
)
parser.add_argument(
    "-Dt",
    "--discovery-topic",
    dest="discovery_topic",
    type=str,
    help="MQTT Discovery topic",
)
parser.add_argument(
    "-D",
    "--discovery",
    dest="discovery",
    type=int,
    help="Enable(1) or disable(0) MQTT discovery",
)
parser.add_argument(
    "-Dh",
    "--hass_discovery",
    dest="hass_discovery",
    type=int,
    help="Enable(1) or disable(0) Home Assistant MQTT discovery (default: 1)",
)
parser.add_argument(
    "-Dn",
    "--discovery_name",
    dest="discovery_device_name",
    type=str,
    help="Device name for Home Assistant",
)
parser.add_argument(
    "-Df",
    "--discovery_filter",
    dest="discovery_filter",
    nargs="+",
    default=[],
    help="Device discovery filter list for Home Assistant",
)
parser.add_argument(
    "-a",
    "--adapter",
    dest="adapter",
    type=str,
    help="Bluetooth adapter (e.g. hci1 on Linux)",
)
parser.add_argument(
    "-s",
    "--scanning_mode",
    dest="scanning_mode",
    type=str,
    choices=("active", "passive"),
    help="Scanning mode (default: active)",
)
parser.add_argument(
    "-ts",
    "--time_sync",
    dest="time_sync",
    nargs="+",
    default=[],
    help="Addresses of Bluetooth devices to synchronize the time",
)
parser.add_argument(
    "-tf",
    "--time_format",
    dest="time_format",
    type=int,
    help="Use 12-hour (1) or 24-hour (0) time format for clocks (default: 0)",
)
args = parser.parse_args()

try:
    with open(conf_path, encoding="utf-8") as config_file:
        config = json.load(config_file)
except Exception:
    config = default_config

# Merge default configuration, with data read from the configuration file
# overriding default data.
# This guarantees that all keys we refer to are in the dictionary.
config = {**default_config, **config}

if args.host:
    config["host"] = args.host
if args.port:
    config["port"] = args.port
if args.user:
    config["user"] = args.user
if args.pwd:
    config["pass"] = args.pwd
if args.pub_topic:
    config["publish_topic"] = args.pub_topic
if args.sub_topic:
    config["subscribe_topic"] = args.sub_topic
if args.publish_all:
    config["publish_all"] = args.publish_all
if args.scan_dur:
    config["ble_scan_time"] = args.scan_dur
if args.time_between:
    config["ble_time_between_scans"] = args.time_between
if args.log_level:
    config["log_level"] = args.log_level

if args.discovery is not None:
    config["discovery"] = args.discovery
elif "discovery" not in config.keys():
    config["discovery"] = default_config["discovery"]
    config["discovery_topic"] = default_config["discovery_topic"]
    config["discovery_device_name"] = default_config["discovery_device_name"]
    config["discovery_filter"] = default_config["discovery_filter"]

if args.hass_discovery is not None:
    config["hass_discovery"] = args.hass_discovery

if args.discovery_topic:
    config["discovery_topic"] = args.discovery_topic
elif "discovery_topic" not in config.keys():
    config["discovery_topic"] = default_config["discovery_topic"]

if args.discovery_device_name:
    config["discovery_device_name"] = args.discovery_device_name
elif "discovery_device_name" not in config.keys():
    config["discovery_device_name"] = default_config["discovery_device_name"]

if args.discovery_filter:
    config["discovery_filter"] = default_config["discovery_filter"]
    if args.discovery_filter[0] != "reset":
        for item in args.discovery_filter:
            config["discovery_filter"].append(item)
elif "discovery_filter" not in config.keys():
    config["discovery_filter"] = default_config["discovery_filter"]

if args.adapter:
    config["adapter"] = args.adapter

if args.scanning_mode:
    config["scanning_mode"] = args.scanning_mode

if args.time_sync:
    config["time_sync"] = default_config["time_sync"]
    if args.time_sync[0] != "reset":
        for item in args.time_sync:
            config["time_sync"].append(item)
elif "time_sync" not in config.keys():
    config["time_sync"] = default_config["time_sync"]

if args.time_format is not None:
    config["time_format"] = args.time_format

if not config["host"]:
    sys.exit("Invalid MQTT host")

try:
    with open(conf_path, mode="w", encoding="utf-8") as config_file:
        config_file.write(json.dumps(config, sort_keys=True, indent=4))
except Exception as exception:
    raise SystemExit("Unable to write config file") from exception

run(conf_path)



# read the contents of your README file
from pathlib import Path

from packaging import version
from skbuild import setup
from skbuild.cmaker import get_cmake_version
from skbuild.exceptions import SKBuildError

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

# Add CMake as a build requirement if cmake is not installed
# or is too low a version
setup_requires = []
try:
    if version.parse(get_cmake_version()) < version.parse("3.4"):
        setup_requires.append("cmake")
except SKBuildError:
    setup_requires.append("cmake")

setup(
    name="TheengsGateway",
    version="v0.8.0",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Theengs",
    url="https://github.com/theengs/gateway",
    license="GPL-3.0 License",
    package_dir={"TheengsGateway": "TheengsGateway"},
    packages=["TheengsGateway"],
    scripts=["bin/TheengsGateway"],
    setup_requires=setup_requires,
    include_package_data=True,
    install_requires=[
        "bleak>=0.15.0",
        "bluetooth-clocks<1.0",
        "paho-mqtt>=1.6.1",
    ],
)
